![MapmyIndia APIs](https://www.mapmyindia.com/api/img/mapmyindia-api.png)

# Flutter MapmyIndia Direction Widget

A ready to use Fragment to show the Routes in an Android platform. It offers the following basic functionalities:

- Takes support of MapmyIndia Place search for searching locations of origin, destinations and via points.
 - It allows to use origin and destinations in MapmyIndia's digital address (semicolon separated) eLoc or WGS 84 geographical coordinates both.
 - The ability to set the vehicle profile like driving, and biking.
 - Easily set the resource for traffic and ETA information.
For more details, please contact apisupport@mapmyindia.com.

## Getting Started
To work with MapmyIndia Direction Widget in flutter add these to your package's pubspec.yaml file:
~~~yaml
dependencies:
  mapmyindia_direction_plugin: ^0.1.0
~~~

Now in your dart code you need to import this package:
~~~dart
import 'package:mapmyindia_direction_plugin/mapmyindia_direction_plugin.dart';
~~~



## Adding MapmyIndia Keys

You **must** provide your keys through the `MapmyIndiaAccountManager` class.([click here](https://github.com/MapmyIndia/flutter-mapmyindia-gl#adding-mapmyindia-keys))

## Direction Widget

Use method `openDirectionWidget` to open Direction Widget:
~~~dart
// Platform messages may fail, so we use a try/catch PlatformException.
try {

DirectionCallbcak directionCallback = await openDirectionWidget(directionOptions:options );
} on PlatformException {

}
~~~
  You can use `DirectionOptions` to set the properties of the widget:
 1.  `resource(String)`:  **Below are the available resource:**
     -   DirectionsCriteria.RESOURCE_ROUTE  **(Default)**: to calculate a route & its duration without considering traffic conditions.
     -   DirectionsCriteria.RESOURCE_ROUTE_ETA get the updated duration of a route considering live traffic; Applicable for India only "region=ind" and "rtype=1" is not supported. This is different from route_traffic; since this doesn't search for a route considering traffic, it only applies delays to the default route.
     -   DirectionsCriteria.RESOURCE_ROUTE_TRAFFIC:
    to search for routes considering live traffic; Applicable for India only “region=ind” and “rtype=1” is not supported
2.  `showAlternative(Boolean)`: Show alternative routes.
3.  `profile(String)`:  **Below are the available profile:**
    -   DirectionsCriteria.PROFILE_DRIVING  **(Default)**:Meant for car routing
    -   DirectionsCriteria.PROFILE_WALKING: Meant for pedestrian routing. Routing with this profile is restricted to route_adv only. region & rtype request parameters are not supported in pedestrian routing
    -   DirectionsCriteria.PROFILE_BIKING:Meant for two-wheeler routing. Routing with this profile is restricted to route_adv only. region & rtype request parameters are not supported in two-wheeler routing.
    -   DirectionsCriteria.PROFILE_TRUCKING:Meant for Truck routing. Routing with this profile is restricted to route_adv only. region & rtype request parameters are not supported in truck routing.
4.  `overview(String)`: Add overview geometry either full, simplified according to highest zoom level it could be display on, or not at all.  **Below are the available value:**
    -   DirectionsCriteria.OVERVIEW_FULL
    -   DirectionsCriteria.OVERVIEW_FALSE
    -   DirectionsCriteria.OVERVIEW_SIMPLIFIED
5.  `steps(Boolean)`: Return route steps for each route leg. Possible values are true/false. By default it will be used as false.
6.  `excludes(List<String>)`  : Additive list of road classes to avoid, order does not matter.  **Below are the available value:**
    -   DirectionsCriteria.EXCLUDE_FERRY
    -   DirectionsCriteria.EXCLUDE_MOTORWAY
    -   DirectionsCriteria.EXCLUDE_TOLL
7.  `showStartNavigation(Boolean)`: To show the Start Navigation button if the origin is current location.
8.  `destination(DirectionPoint)`: You can use  `DirectionPoint`  to pass the destination in direction widget:
    -   `location(LatLng)`: Latitude and longitude of destination
    -   `eLoc(String)`: It takes eloc
    - `placeName(String)`: It takes the place name
    -  `placeAddress(String)`: It takes the address
  9. `searchPlaceOption(PlaceOptions`): To set the properties of search widget


![Email](https://www.google.com/a/cpanel/mapmyindia.co.in/images/logo.gif?service=google_gsuite)

Email us at [apisupport@mapmyindia.com](mailto:apisupport@mapmyindia.com)

​

![](https://www.mapmyindia.com/api/img/icons/stack-overflow.png)

[Stack  Overflow](https://stackoverflow.com/questions/tagged/mapmyindia-api)

Ask a question under the mapmyindia-api

​

![](https://www.mapmyindia.com/api/img/icons/support.png)

[Support](https://www.mapmyindia.com/api/index.php#f_cont)

Need support? contact us!

​

![](https://www.mapmyindia.com/api/img/icons/blog.png)

[Blog](http://www.mapmyindia.com/blog/)

Read about the latest updates & customer stories

​

​

>  ©  Copyright  2021.  CE  Info  Systems  Pvt.  Ltd.  All  Rights  Reserved.  |  [Terms  &  Conditions](http://www.mapmyindia.com/api/terms-&-conditions).
