package com.mapmyindia.sdk.flutter.direction

import android.app.Activity
import android.content.Intent
import androidx.annotation.NonNull
import com.mapbox.mapboxsdk.MapmyIndia

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry

/** MapmyindiaDirectionPlugin */
class MapmyindiaDirectionPlugin : FlutterPlugin, MethodCallHandler, ActivityAware, PluginRegistry.ActivityResultListener {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel
    private lateinit var result: Result
    private var activity: Activity? = null
    private var binding: ActivityPluginBinding? = null

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "mapmyindia_direction_plugin")
        channel.setMethodCallHandler(this)
        MapmyIndia.getInstance(flutterPluginBinding.applicationContext)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        if (call.method == "openDirectionWidget") {
            if (activity != null) {
                this.result = result
                val intent = Intent(activity!!, DirectionActivity::class.java)
                if(call.hasArgument("directionOptions")) {
                    intent.putExtra("com.mapmyindia.sdk.flutter.direction.DIRECTION_OPTION", Utils.getDirectionOption(call.argument<Map<String, Any?>>("directionOptions") as Map<String, Any?>))
                }
                activity?.startActivityForResult(intent, 13)
                binding?.addActivityResultListener(this)
            } else {
                result.error("Failure", "Not attached to activity", null)

            }
        } else {
            result.notImplemented()
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        this.activity = binding.activity
        this.binding = binding;
    }

    override fun onDetachedFromActivityForConfigChanges() {
        this.activity = null
        this.binding = null
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        this.activity = binding.activity
        this.binding = binding
    }

    override fun onDetachedFromActivity() {
        this.activity = null
        this.binding = null
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
        if (requestCode == 13) {
            if (resultCode == Activity.RESULT_OK) {

                val selectedIndex = MapmyIndiaDirectionsSettings.getInstance().selectedIndex

                val directionsResponse = MapmyIndiaDirectionsSettings.getInstance().directionsResponse
                MapmyIndiaDirectionsSettings.getInstance().clear()


                val resultMap: MutableMap<String, Any?> = mutableMapOf()

                resultMap.put("selectedIndex", selectedIndex ?: 0)
                resultMap.put("directionsResponse", directionsResponse?.toJson())
                this.result.success(resultMap)
                binding?.removeActivityResultListener(this)
            }
        }
        return true
    }
}