package com.mapmyindia.sdk.flutter.direction

import android.graphics.Color
import com.mapbox.geojson.Point
import com.mapmyindia.sdk.direction.ui.model.DirectionOptions
import com.mapmyindia.sdk.direction.ui.model.DirectionPoint
import com.mapmyindia.sdk.plugins.places.autocomplete.model.PlaceOptions

/**
 ** Created by Saksham on 26-04-2021.
 **/
object Utils {
    fun getDirectionOption(map: Map<String, Any?>): DirectionOptions {
        val builder: DirectionOptions.Builder = DirectionOptions.builder()
        builder.theme(DirectionOptions.THEME_DAY)
        if(map.containsKey("showAlternative")) {
            builder.showAlternative(map["showAlternative"] as Boolean)
        }
        if(map.containsKey("showProfileOption")) {
            builder.showProfileOption(map["showProfileOption"] as Boolean)
        }
        if(map.containsKey("showStartNavigation")) {
            builder.showStartNavigation(map["showStartNavigation"] as Boolean)
        }
        if(map.containsKey("resource")) {
            builder.resource(map["resource"] as String)
        }
        if(map.containsKey("profile")) {
            builder.profile(map["profile"] as String)
        }
        if(map.containsKey("steps")) {
            builder.steps(map["steps"] as Boolean)
        }
        if(map.containsKey("overview")) {
            builder.overview(map["overview"] as String)
        }
        if(map.containsKey("theme")) {
            builder.theme(map["theme"] as Int)
        }
        if(map.containsKey("annotation")) {
            builder.annotation(map["annotation"] as List<String>)
        }
        if(map.containsKey("excludes")) {
            builder.excludes(map["excludes"] as List<String>)
        }
        if(map.containsKey("searchPlaceOption")) {
            builder.searchPlaceOption(getPlaceOptions(map["searchPlaceOption"] as Map<String, Any?>))
        }
        if(map.containsKey("destination")) {
            builder.destination(getDirectionPoint(map["destination"] as Map<String, Any?>))
        }

        return builder.build()
    }

    fun getDirectionPoint(map: Map<String, Any?>): DirectionPoint {
        if(map.containsKey("location")) {
            val locations: List<Double> = map["location"] as List<Double>
//            if(locations[0] != null && locations[1] != null) {
            return DirectionPoint.setDirection(Point.fromLngLat(locations[1], locations[0]), map["placeName"] as String, map["placeAddress"] as String)
//            }
        }
        return DirectionPoint.setDirection(map["eLoc"] as String, map["placeName"] as String, map["placeAddress"] as String)
    }

    fun getPlaceOptions(params: Map<String, Any?>): PlaceOptions {
        val builder: PlaceOptions.Builder = PlaceOptions.builder()
        if(params.containsKey("enableTextSearch")) {
            builder.enableTextSearch(params["enableTextSearch"] as Boolean?)
        }
        if(params.containsKey("filter")) {
            builder.filter(params["filter"] as String?)
        }
        if(params.containsKey("hint")) {
            builder.hint(params["hint"] as String?)
        }
        if(params.containsKey("historyCount")) {
            builder.historyCount(params["historyCount"] as Int?)
        }
        if(params.containsKey("limit")) {
            builder.limit(params["limit"] as Int)
        }
        if(params.containsKey("pod")) {
            builder.pod(params["pod"] as String?)
        }
        if(params.containsKey("saveHistory")) {
            builder.saveHistory(params["saveHistory"] as Boolean?)
        }
        if(params.containsKey("backgroundColor")) {
            val color = params["backgroundColor"] as String
            builder.backgroundColor(Color.parseColor(color))
        } else {
            builder.backgroundColor(Color.WHITE)
        }
        if(params.containsKey("statusbarColor")) {
            val color = params["statusbarColor"] as String
            builder.statusBarColor(Color.parseColor(color))
        }
        if(params.containsKey("toolbarColor")) {
            val color = params["toolbarColor"] as String
            builder.toolbarColor(Color.parseColor(color))
        }
        if(params.containsKey("toolbarTintColor")) {
            val color = params["toolbarTintColor"] as String
            builder.toolbarTintColor(Color.parseColor(color))
        }
        if(params.containsKey("tokenizeAddress")) {
            builder.tokenizeAddress(params["tokenizeAddress"] as Boolean?)
        }
        if(params.containsKey("userAddedLocationEnable")) {
            builder.userAddedLocationEnable(params["userAddedLocationEnable"] as Boolean?)
        }
        if(params.containsKey("zoom")) {
            builder.zoom(params["zoom"] as Double?)
        }
        if(params.containsKey("location")) {
            val locations: List<Double> = params["location"] as List<Double>
            builder.location(Point.fromLngLat(locations[1], locations[0]))
        }
        return builder.build();

    }
}