//
//  MapmyIndiaDirectionsConfiguration.swift
//  mapmyindia_direction_plugin
//
//  Created by CE00120420 on 28/04/21.
//

import UIKit
import MapmyIndiaDirectionsUI
import MapmyIndiaDirections
import MapmyIndiaUIWidgets
import MapmyIndiaMaps

class MapmyIndiaDirectionsConfiguration: NSObject {
    var directionUI = MapmyIndiaDirectionsViewController()
    var argsList : [String: Any]!
    var directionLocations: [MapmyIndiaDirectionsLocation] = [MapmyIndiaDirectionsLocation]()
    func setupDirectionUIConfiguration(argsList: [String:Any], directionUI: MapmyIndiaDirectionsViewController) {
        self.directionUI = directionUI
        self.argsList = argsList
        if argsList.keys.contains("directionOptions") {
            if let args = argsList["directionOptions"] as? [String: Any] {
                setupDirectionsParameter(args: args)
            }
            if argsList.keys.contains("") {
                if let args = argsList["directionOptions"] as? [String: Any] {
                    setupDirectionsParameter(args: args)
                }
            }
        }
        print(argsList)
    }

    func setupDirectionsParameter(args: [String: Any]) {
        
        if args.keys.contains("showAlternative") {
            directionUI.isIncludesAlternativeRoutes = args["showAlternative"] as? Bool ?? true
        }
        
        if args.keys.contains("showProfileOption") {

        }
        
        if args.keys.contains("annotation") {
            var attributionArray: MBAttributeOptions = MBAttributeOptions()
            if let attr = args["annotation"] as? [String] {
                attributionArray = AttributeOptions.init(descriptions: attr)!
                print(attributionArray)
            }
            directionUI.attributationOptions = attributionArray
        }
        
        if args.keys.contains("showStartNavigation") {
            directionUI.isShowStartNavigation = args["showStartNavigation"] as? Bool ?? true
        }
        
        if args.keys.contains("resource") {
            if let resRawValue = args["resource"] as? String {
                let resourceIdentifier = MBDirectionsResourceIdentifier.init(resRawValue)
                directionUI.resourceIdentifier = resourceIdentifier
            }
        }

        if args.keys.contains("profile") {
            if let resRawValue = args["profile"] as? String {
                let resourceIdentifier = MBDirectionsProfileIdentifier.init(resRawValue)
                directionUI.profileIdentifier = resourceIdentifier
            }
        }
        
        if args.keys.contains("steps") {
            directionUI.includesSteps = args["steps"] as? Bool ?? true
        }
        
        if args.keys.contains("overview") {
            if let rawValue = args["overview"] as? UInt {
                let res = RouteShapeResolution.init(rawValue: rawValue)
                directionUI.routeShapeResolution = res ?? .full
            }
        }
        
        if args.keys.contains("theme") {
//            switch args["theme"] as! Int {
//            case 0:
//                Styles.init().style = DefaultStyle.init()
//            case 1:
//                Styles.init().style = DayStyle.init()
//            case 2:
//                Styles.init().style = NightStyle.init()
//            default:
//                Styles.init().style = DefaultStyle.init()
//            }
//
//            Styles.init().style = NightStyle.init()
        }
        
        if args.keys.contains("excludes") {
            var roadClassesToAvoid: RoadClasses = RoadClasses()
            if let attr = args["excludes"] as? [String] {
                roadClassesToAvoid = RoadClasses.init(descriptions: attr)!
                print(roadClassesToAvoid)
            }
//            directionUI.roadClassesToAvoid = roadClassesToAvoid
        }
        
        if args.keys.contains("searchPlaceOption") {
            let autoCompleteVC = MapmyIndiaAutocompleteViewController()
            if let searchPlaceOptions = args["searchPlaceOption"] as? [String: Any] {
                let auto = getPlaceOptions(argsLists: searchPlaceOptions, autoCompleteVC: autoCompleteVC)
                directionUI.autocompleteViewController = auto
            }
        }
    }
    
    func getPlaceOptions(argsLists: [String: Any], autoCompleteVC: MapmyIndiaAutocompleteViewController) -> MapmyIndiaAutocompleteViewController {
        let   list = (argsLists)
        let autocompleteFilter = MapmyIndiaAutocompleteFilter()
        
        if list.keys.contains("saveHistory") {
            IsHistoryEnabled = list["saveHistory"] as! Bool
        }
        
        if list.keys.contains("primaryTextColor") {
            autoCompleteVC.primaryTextColor = hexStringToUIColor(hex: list["primaryTextColor"] as! String)
        }
        
        if list.keys.contains("tintColor") {
            autoCompleteVC.tintColor = hexStringToUIColor(hex: list["tintColor"] as! String)
        }
        
        if list.keys.contains("tableCellSeparatorColor") {
            autoCompleteVC.tableCellSeparatorColor = hexStringToUIColor(hex: list["tableCellSeparatorColor"] as! String)
        }
        
        if list.keys.contains("primaryTextHighlightColor") {
            autoCompleteVC.primaryTextHighlightColor = hexStringToUIColor(hex: list["primaryTextHighlightColor"] as! String)
        }
        
        if list.keys.contains("historyCount") {
            HistoryCountToShow = list["historyCount"] as! Int
        }
        if list.keys.contains("enableTextSearch") {
            IsTextSearchEnabledOnPressingEnterOrSearchKey = list["enableTextSearch"] as! Bool
        }
        if(list.keys.contains("location")) {
            let location = list["location"] as! [Double]
            autocompleteFilter.origin = CLLocation(latitude: location[0], longitude:location[1])
            autoCompleteVC.autocompleteFilter = autocompleteFilter
        }
        if(list.keys.contains("zoom")) {
            autocompleteFilter.zoom = list["zoom"] as? NSNumber
            autoCompleteVC.autocompleteFilter = autocompleteFilter
        }
        if(list.keys.contains("tokenizeAddress")) {
            autocompleteFilter.attributions = list["tokenizeAddress"] as! Bool
            autoCompleteVC.autocompleteFilter = autocompleteFilter
        }
        
        if(list.keys.contains("pod")) {
            autocompleteFilter.resultPlaceType = MMIPodTypeIdentifier(list["pod"] as! String)
            autoCompleteVC.autocompleteFilter = autocompleteFilter
        }
        
        if list.keys.contains("backgroundColor"){
            autoCompleteVC.tableCellBackgroundColor = hexStringToUIColor(hex: list["backgroundColor"] as! String)
        }
        if list.keys.contains("toolbarColor") {
            let color = hexStringToUIColor(hex: list["toolbarColor"] as! String)
            UINavigationBar.appearance(whenContainedInInstancesOf: [MapmyIndiaAutocompleteViewController.self]).barTintColor = color
        }
        
        if list.keys.contains("searchTextColor") {
            let color = hexStringToUIColor(hex: list["searchTextColor"] as! String)
            let searchBarTextAttributes = [
                NSAttributedString.Key.foregroundColor: color,
                NSAttributedString.Key.font: UIFont.systemFont(ofSize: UIFont.systemFontSize)
            ]
            UITextField.appearance(whenContainedInInstancesOf: [MapmyIndiaAutocompleteViewController.self]).defaultTextAttributes = searchBarTextAttributes
        }
        
        autoCompleteVC.attributionSettings = mapmyIndiaAttributionsSettings(options: list)
        
        if list.keys.contains("hint") {
            let placeHolderText = list["hint"] as! String
            UITextField.appearance(whenContainedInInstancesOf: [MapmyIndiaAutocompleteViewController.self]).placeholder = placeHolderText
        }
        
        return autoCompleteVC
    }
    
    func autocompleteFilter(options: [String: Any]) -> MapmyIndiaAutocompleteFilter {
        let autocompleteFilter = MapmyIndiaAutocompleteFilter()
        if(options.keys.contains("location")) {
            let location = options["location"] as! [Double]
            autocompleteFilter.origin = CLLocation(latitude: location[0], longitude:location[1])
        }
        if(options.keys.contains("zoom")) {
            autocompleteFilter.zoom = options["zoom"] as? NSNumber
        }
        if(options.keys.contains("tokenizeAddress")) {
            autocompleteFilter.attributions = options["tokenizeAddress"] as! Bool
        }
        
        if(options.keys.contains("pod")) {
            autocompleteFilter.resultPlaceType = MMIPodTypeIdentifier(options["pod"] as! String)
        }
        return autocompleteFilter
    }
    
    func fromDict(_ dict: [String: Any], mapView: MapmyIndiaMapView) -> MGLMapCamera? {
        guard let target = dict["target"] as? [Double],
              let zoom = dict["zoom"] as? Double,
              let tilt = dict["tilt"] as? CGFloat,
              let bearing = dict["bearing"] as? Double else { return nil }
        let location = CLLocationCoordinate2D(latitude: target[0], longitude: target[1])
        let altitude = MGLAltitudeForZoomLevel(zoom, tilt, location.latitude, mapView.frame.size)
        return MGLMapCamera(lookingAtCenter: location, altitude: altitude, pitch: tilt, heading: bearing)
    }
    
    func mapmyIndiaAttributionsSettings(options: [String: Any]) -> MapmyIndiaAttributionsSettings {
        let attribution = MapmyIndiaAttributionsSettings()
        
        if options.keys.contains("attributionHorizontalAlignment") {
            switch options["attributionHorizontalAlignment"] as! Int {
            case 1:
                attribution.attributionHorizontalContentAlignment = .left
            case 2:
                attribution.attributionHorizontalContentAlignment = .center
            case 3:
                attribution.attributionHorizontalContentAlignment = .right
            default:
                attribution.attributionHorizontalContentAlignment = .center
            }
        }
        
        if options.keys.contains("attributionVerticalAlignment") {
            switch options["attributionVerticalAlignment"] as! Int {
            case 4:
                attribution.attributionVerticalPlacement = .before
            case 5:
                attribution.attributionVerticalPlacement = .after
            default:
                attribution.attributionVerticalPlacement = .before
            }
        }
        
        if options.keys.contains("logoSize") {
            switch options["logoSize"] as! Int {
            case 6:
                attribution.attributionSize = .small
            case 7:
                attribution.attributionSize = .medium
            case 8:
                attribution.attributionSize = .large
            default:
                attribution.attributionSize = .small
            }
        }
        return attribution
    }
    
    func hexStringToUIColor (hex:String) -> UIColor {
        var cString:String = hex.trimmingCharacters(in: .whitespacesAndNewlines).uppercased()
        
        if (cString.hasPrefix("#")) {
            cString.remove(at: cString.startIndex)
        }
        
        if ((cString.count) != 6) {
            return UIColor.gray
        }
        
        var rgbValue:UInt64 = 0
        Scanner(string: cString).scanHexInt64(&rgbValue)
        
        return UIColor(
            red: CGFloat((rgbValue & 0xFF0000) >> 16) / 255.0,
            green: CGFloat((rgbValue & 0x00FF00) >> 8) / 255.0,
            blue: CGFloat(rgbValue & 0x0000FF) / 255.0,
            alpha: CGFloat(1.0)
        )
    }
}

