import Flutter
import UIKit
import MapmyIndiaDirectionsUI
import MapmyIndiaDirections
import Polyline
public class SwiftMapmyindiaDirectionPlugin: NSObject, FlutterPlugin {
    var result : FlutterResult!
    var flutterViewController: FlutterViewController!
    var directionUI: MapmyIndiaDirectionsViewController!
    var directionLocations = [MapmyIndiaDirectionsLocation]()
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "mapmyindia_direction_plugin", binaryMessenger: registrar.messenger())
        let instance = SwiftMapmyindiaDirectionPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if call.method == "openDirectionWidget" {
            self.flutterViewController = UIApplication.shared.delegate?.window??.rootViewController as? FlutterViewController
            
            let directionConfig = MapmyIndiaDirectionsConfiguration()
            var argsLists: [String: Any]
            if let args = call.arguments {
                argsLists = (args as? [String : Any] ?? [String: Any]())
                directionLocations.removeAll()
                if let directionOptions = argsLists["directionOptions"] as? [String: Any] {
                    setupDirectionsParameter(args: directionOptions)
                }
                directionUI = MapmyIndiaDirectionsViewController(for: directionLocations)
                
                let _ = directionConfig.setupDirectionUIConfiguration(argsList: argsLists, directionUI: directionUI)
            }
            directionUI.modalPresentationStyle = .fullScreen
            directionUI.delegate = self
            flutterViewController.present(directionUI, animated: false, completion: nil)
            self.result = result
        }
    }
    
    func setupDirectionsParameter(args: [String: Any]) {
        
        if args.keys.contains("destination") {
            if let locationDestination = args["destination"] as? [String:Any] {
                if let destination =  addLocations(locations: locationDestination, locationType: .suggestion) {
                    directionLocations.append(destination)
                }
            }
        }
        if args.keys.contains("source") {
            if let locationSource = args["source"] as? [String:Any] {
                if let source = addLocations(locations: locationSource, locationType: .suggestion) {
                    directionLocations.insert(source, at: 0)
                }
            }
        }
    }
    
    func addLocations(locations: [String:Any], locationType: MapmyIndiaDirectionsLocationType) -> MapmyIndiaDirectionsLocation? {
        var directionLocation : MapmyIndiaDirectionsLocation?
        var location: String?
        var placeAddress: String?
        var placeName: String?
        if locations.keys.contains("location") {
            let latLong = locations["location"] as! [Double]
            var latitude: String!
            var logitude: String!
            if let lat = latLong.first , let long = latLong.last {
                latitude = String(lat)
                logitude = String(long)
                location = "\(logitude!),\(latitude!)"
                print(location!)
            }
        }
        
        if locations.keys.contains("eLoc") {
            location = locations["eLoc"] as? String
            print(location ?? "")
        }
        if locations.keys.contains("placeAddress") {
            placeAddress = locations["placeAddress"] as? String
        }
        
        if locations.keys.contains("placeName") {
            placeName = locations["placeName"] as? String
        }
        
        directionLocation = MapmyIndiaDirectionsLocation(location:"\(location ?? "")", displayName: placeName ?? "placeName" , displayAddress: placeAddress ?? "placeAddress", locationType: locationType)
        return directionLocation
    }
    
    
}

extension SwiftMapmyindiaDirectionPlugin: MapmyIndiaDirectionsViewControllerDelegate {
    public func didRequestForGoBack(for view: MapmyIndiaDirectionsTopBannerView) {
        flutterViewController.dismiss(animated: false, completion: nil)
    }
    
    public func didRequestForAddViapoint(_ sender: UIButton, for isEditMode: Bool) {
        
    }
    
    public func didRequestDirections() {
        
    }
    
    public func didRequestForStartNavigation(for routes: [Route], locations: [MapmyIndiaDirectionsLocation], selectedRouteIndex: Int, error: NSError) {
        let returnDictionary = NSMutableDictionary()
        
        returnDictionary["selectedIndex"] = selectedRouteIndex
        var dict: [String: Any?] = ["code": "OK"]
                var routeList: [Any] = [Any]()
                var wayPointList: [Any] = [Any]()
                
                        
               for location in locations {
                if let coor = location.location?.split(separator: ",") {
                    if coor.count > 1 {
                        let lat = coor.first
                        let long = coor.last
                        var data: [String: Any?] = [
                            "location": [Double(long!),Double(lat!)],
                                    "name": location.displayName
                                ]
                                wayPointList.append(data)                    }
                }
               
                                }
                dict["waypoints"] = wayPointList
                for item in routes {
                    
                        var data: [String: Any?] = [
                            "distance": item.distance,
                        "duration": item.expectedTravelTime,
                            "geometry": Polyline(coordinates: item.coordinates ?? [], precision: 1e6).encodedLevels,
                        "routeIndex": routes.index(of: item)
                    ]
                    if item.legs != nil {
                        var legList: [Any] = [Any]()
                        for legs in item.legs {
                            var legsDict: [String: Any?] = [
                                "distance": legs.distance,
                                "duration": legs.expectedTravelTime,
                                "summary": legs.description
                            ]
                            var stepsList: [Any] = [Any]()
                            for steps in legs.steps {
                                var stepsDict: [String: Any?] = [
                                    "distance": steps.distance,
                                    "duration": steps.expectedTravelTime,
                                    "geometry": Polyline(coordinates: steps.coordinates ?? [], precision: 1e6).encodedLevels,
                                    "name": steps.names?.count ?? 0 > 0 ? steps.names![0]: nil,
                                    "destinations": steps.destinations?.count ?? 0 > 0 ? steps.destinations![0]: nil,
                                    "mode": steps.transportType.description,
                                    "pronunciation": steps.instructions,
                                    "rotary_name": steps.exitNames?.count ?? 0 > 0 ? steps.exitNames![0]: nil,
                                    "rotary_pronunciation": steps.phoneticExitNames?.count ?? 0 > 0 ? steps.phoneticExitNames![0]: nil,
                                    "driving_side": steps.drivingSide.description,
                                    "maneuver": [
        //                                "degree": steps.,
                                        "location": [steps.maneuverLocation.longitude, steps.maneuverLocation.latitude],
                                        "bearing_before": steps.initialHeading,
                                        "bearing_after": steps.finalHeading,
                                        "instruction": steps.instructions,
                                        "type": steps.maneuverType.description,
                                        "modifier": steps.maneuverDirection.description,
                                        "exit": steps.exitIndex
                                    ]
                                ]
                                if let voiceInstruction = steps.instructionsSpokenAlongStep {
                                    var voiceInstuctionList = [Any]()
                                    for spokenInstruction in voiceInstruction {
                                        let spokenDict: [String: Any?] = [
                                            "distanceAlongGeometry": spokenInstruction.distanceAlongStep,
                                            "announcement": spokenInstruction.text,
                                            "ssmlAnnouncement": spokenInstruction.ssmlText
                                        ]
                                        voiceInstuctionList.append(spokenDict)
                                    }
                                    stepsDict["voiceInstructions"] = voiceInstuctionList
                                }
                                if let intersections = steps.intersections {
                                    var intersectionList = [Any]()
                                    for stepsintersection in intersections {
                                        var intersectionDict: [String: Any?] = [
                                            "location": [stepsintersection.location.longitude, stepsintersection.location.latitude],
                                            "bearings": stepsintersection.headings,
                                            "classes": stepsintersection.outletRoadClasses,
                                            "entry": stepsintersection.outletIndexes,
                                            "in": stepsintersection.approachIndex,
                                            "out": stepsintersection.outletIndex
                                        ]
                                        if let lanes = stepsintersection.approachLanes {
                                            var lanesList = [Any]()
                                            for laneItem in lanes {
                                                lanesList.append([
        //                                            "valid": laneItem.
                                                    "indications": laneItem.indications
                                                ])
                                            }
                                            intersectionDict["lanes"] = lanesList
                                        }
                                    }
                                    stepsDict["intersections"] = intersectionList
                                }
                                
                                stepsList.append(stepsDict)
                            }
                            legsDict["steps"] = stepsList
                            legsDict["annotation"] = [
                                "distance": legs.segmentDistances,
                                "duration": legs.expectedSegmentTravelTimes,
                                "speed": legs.segmentSpeeds,
                                "congestion": legs.segmentCongestionLevels?.map { $0.description }
        //                        "baseDuration": legs.base
        //                        "nodes": legs.
        //                        "maxspeed": legs.segmen
                            ]
                            legList.append(legsDict)
                        }
                        data["legs"] = legList
                    }
                    
                    var coordinates: [String] = [String]()
                    for waypoint in item.routeOptions.waypoints {
                        if(waypoint.coordinate != nil) {
                            coordinates.append("\(waypoint.coordinate.longitude),\(waypoint.coordinate.latitude)")
                        } else {
                            coordinates.append(waypoint.eLoc!)
                        }
                    }
                    var routeOption: [String: Any?] = [
        //                "deviceID": item.routeOptions.
        //                "user": item.routeOptions.user
                        "profile": item.routeOptions.profileIdentifier.rawValue,
                        "resource": item.routeOptions.resourceIdentifier.rawValue,
                        "coordinates": coordinates,
                        "alternatives": item.routeOptions.includesAlternativeRoutes,
        //                "radiuses": item.routeOptions.rad
        //                "bearings": item.routeOptions.
        //                "language": item.routeOptions.lan
        //                "lessverbose": item.routeOptions.less
                        "geometries": item.routeOptions.shapeFormat.description,
                        "overview": item.routeOptions.routeShapeResolution.description,
                        "steps": item.routeOptions.includesSteps,
                        "annotations": item.routeOptions.attributeOptions.description,
                        "exclude": item.routeOptions.roadClassesToAvoid.description,
                    ]
                    
                    data["routeOptions"] = routeOption
                    routeList.append(data)
                }
                dict["routes"] = routeList
        do {
            let jsonData = try JSONSerialization.data(withJSONObject: dict)
            if let dictFromJSON = String(data: jsonData, encoding: .utf8) {
                returnDictionary["directionsResponse"] = dictFromJSON
                self.result(returnDictionary)
            }
        } catch {
            print(error.localizedDescription)
        }
        UIApplication.shared.keyWindow?.rootViewController?.dismiss(animated: false, completion: nil)
    }
    
    public func didRequestForPreviewRoute(for steps: [RouteStep]?) {
        if let steps = steps {
            for i in steps {
            }
        }
    }
}
extension Encodable {
    var dictionary: [String: Any]? {
        guard let data = try? JSONEncoder().encode(self) else { return nil }
        return (try? JSONSerialization.jsonObject(with: data, options: .allowFragments)).flatMap { $0 as? [String: Any] }
    }
}
